<?php
/**
 * @version    CVS: 1.0.0
 * @package    Com_Raffle
 * @author     David George <david.george@bluelinetechnology.com.au>
 * @copyright  2022 David George
 * @license    GNU General Public License version 2 or later; see LICENSE.txt
 */

namespace Blueline\Component\Raffle\Site\Model;
// No direct access.
defined('_JEXEC') or die;

use \Joomla\CMS\Factory;
use \Joomla\CMS\Language\Text;
use \Joomla\CMS\MVC\Model\ListModel;
use \Joomla\Component\Fields\Administrator\Helper\FieldsHelper;
use \Joomla\CMS\Helper\TagsHelper;
use \Joomla\CMS\Layout\FileLayout;
use \Joomla\Database\ParameterType;
use \Joomla\Utilities\ArrayHelper;
use \Blueline\Component\Raffle\Site\Helper\RaffleHelper;


/**
 * Methods supporting a list of Raffle records.
 *
 * @since  1.0.0
 */
class RafflesModel extends ListModel
{
	/**
	 * Constructor.
	 *
	 * @param   array  $config  An optional associative array of configuration settings.
	 *
	 * @see    JController
	 * @since  1.0.0
	 */
	public function __construct($config = array())
	{
		if (empty($config['filter_fields']))
		{
			$config['filter_fields'] = array(
				'id', 'a.id',
				'state', 'a.state',
				'ordering', 'a.ordering',
				'created_by', 'a.created_by',
				'modified_by', 'a.modified_by',
				'name', 'a.name',
				'email', 'a.email',
				'number_of_tickets', 'a.number_of_tickets',
			);
		}

		parent::__construct($config);
	}

	
       /**
        * Checks whether or not a user is manager or super user
        *
        * @return bool
        */
        public function isAdminOrSuperUser()
        {
            try{
                $user = Factory::getUser();
                return in_array("8", $user->groups) || in_array("7", $user->groups);
            }catch(\Exception $exc){
                return false;
            }
        }

	/**
	 * Method to auto-populate the model state.
	 *
	 * Note. Calling getState in this method will result in recursion.
	 *
	 * @param   string  $ordering   Elements order
	 * @param   string  $direction  Order direction
	 *
	 * @return  void
	 *
	 * @throws  Exception
	 *
	 * @since   1.0.0
	 */
	protected function populateState($ordering = null, $direction = null)
	{
		// List state information.
		parent::populateState("a.id", "ASC");

		$app = Factory::getApplication();
		$list = $app->getUserState($this->context . '.list');

		$value = $app->input->get('limit', $app->get('list_limit', 0), 'uint');
		$this->setState('list.limit', $value);

		$value = $app->input->get('limitstart', 0, 'uint');
		$this->setState('list.start', $value);

		$ordering  = $this->getUserStateFromRequest($this->context .'.filter_order', 'filter_order', "a.id");
		$direction = strtoupper($this->getUserStateFromRequest($this->context .'.filter_order_Dir', 'filter_order_Dir', "ASC"));
		
		if(!empty($ordering) || !empty($direction))
		{
			$list['fullordering'] = $ordering . ' ' . $direction;
		}

		$app->setUserState($this->context . '.list', $list);

		

		$context = $this->getUserStateFromRequest($this->context.'.filter.search', 'filter_search');
		$this->setState('filter.search', $context);

		// Split context into component and optional section
		$parts = FieldsHelper::extract($context);

		if ($parts)
		{
			$this->setState('filter.component', $parts[0]);
			$this->setState('filter.section', $parts[1]);
		}
	}

	/**
	 * Build an SQL query to load the list data.
	 *
	 * @return  DatabaseQuery
	 *
	 * @since   1.0.0
	 */
	protected function getListQuery()
	{
			// Create a new query object.
			$db    = $this->getDbo();
			$query = $db->getQuery(true);

			// Select the required fields from the table.
			$query->select(
						$this->getState(
								'list.select', 'DISTINCT a.*'
						)
				);

			$query->from('`#__raffle` AS a');
			
		// Join over the users for the checked out user.
		$query->select('uc.name AS uEditor');
		$query->join('LEFT', '#__users AS uc ON uc.id=a.checked_out');

		// Join over the created by field 'created_by'
		$query->join('LEFT', '#__users AS created_by ON created_by.id = a.created_by');

		// Join over the created by field 'modified_by'
		$query->join('LEFT', '#__users AS modified_by ON modified_by.id = a.modified_by');
		if(!$this->isAdminOrSuperUser()){
			$query->where("a.created_by = " . Factory::getUser()->get("id"));
		}
			
		if (!Factory::getUser()->authorise('core.edit', 'com_raffle'))
		{
			$query->where('a.state = 1');
		}
		else
		{
			$query->where('(a.state IN (0, 1))');
		}
		
		// Filter by search 
        $search = $this->getState('filter.search');
        
        $parts = explode(" ", $search); 

        if (!empty($search))
        {
            if (stripos($search, 'id:') === 0)
            {
                $searchid = (int) substr($search, 3);
                $query->where('a.id = :searchid')->bind(':searchid', $searchid, ParameterType::INTEGER);
            }
            else
            {
                foreach($parts as $key => $part) { 
                    // Where conditions to be built 
                    $conditions = array();
                    
                    // Feilds to compare each part of the search query to 
                    $wheres = array('a.name', 'a.email');
                    
                    // Loop over each where condition to populate the query and bind the variable
                    foreach($wheres as $where) {
                        $varName = $db->escape(':' . preg_replace('/[^A-Za-z0-9\-]/', '', $where) . $key);
                        $varValueName = $db->escape(str_replace(".", '', $where) . $key);
                        ${$varValueName} = '%' . $part . '%';
                        $conditions[] = $where . ' LIKE ' . $varName; 
                        $query->bind($varName, ${$varValueName}, ParameterType::STRING);
                    }
                    $query->where('(' . implode(" OR ", $conditions) . ')');
                }
            }
        }

		// Filtering number_of_tickets
		$filter_number_of_tickets = $this->state->get("filter.number_of_tickets");

		if ($filter_number_of_tickets !== null && (is_numeric($filter_number_of_tickets) || !empty($filter_number_of_tickets)))
		{
			$query->where("a.`number_of_tickets` = :number_of_tickets");
			$query->bind(':number_of_tickets', $filter_number_of_tickets, ParameterType::INTEGER);
		}

			
		/**
		// Add the list ordering clause.
		$orderCol  = $this->state->get('list.ordering', "a.id");
		$orderDirn = $this->state->get('list.direction', "ASC");

		if ($orderCol && $orderDirn)
		{
			$query->order($db->escape($orderCol . ' ' . $orderDirn));
		}
*/

        $query->order($db->escape('-LOG(1.0 - RAND()) / a.`number_of_tickets`'));

		return $query;
	}

	/**
	 * Method to get an array of data items
	 *
	 * @return  mixed An array of data on success, false on failure.
	 */
	public function getItems()
	{
		$items = parent::getItems();
		
		foreach ($items as $item)
		{
				$item->number_of_tickets = empty($item->number_of_tickets) ? '' : Text::_('COM_RAFFLE_RAFFLES_NUMBER_OF_TICKETS_OPTION_' . strtoupper($item->number_of_tickets));
		}

		return $items;
	}

	/**
	 * Overrides the default function to check Date fields format, identified by
	 * "_dateformat" suffix, and erases the field if it's not correct.
	 *
	 * @return void
	 */
	protected function loadFormData()
	{
		$app              = Factory::getApplication();
		$filters          = $app->getUserState($this->context . '.filter', array());
		$error_dateformat = false;

		foreach ($filters as $key => $value)
		{
			if (strpos($key, '_dateformat') && !empty($value) && $this->isValidDate($value) == null)
			{
				$filters[$key]    = '';
				$error_dateformat = true;
			}
		}

		if ($error_dateformat)
		{
			$app->enqueueMessage(Text::_("COM_RAFFLE_SEARCH_FILTER_DATE_FORMAT"), "warning");
			$app->setUserState($this->context . '.filter', $filters);
		}

		return parent::loadFormData();
	}

	/**
	 * Checks if a given date is valid and in a specified format (YYYY-MM-DD)
	 *
	 * @param   string  $date  Date to be checked
	 *
	 * @return bool
	 */
	private function isValidDate($date)
	{
		$date = str_replace('/', '-', $date);
		return (date_create($date)) ? Factory::getDate($date)->format("Y-m-d") : null;
	}
	
	public function getStats() {
    	$return = new \stdClass; 
    	$return->total_tickets = 0; 
    	$return->total_raised = 0; 
    	
    	$db    = $this->getDbo();
    	
		$query = $db->getQuery(true);
        $query->select('SUM(number_of_tickets) as total_tickets');
		$query->from('`#__raffle` AS a');
		
		$db->setQuery($query); 
		
		if($result = $db->loadObject()) {
		    $return->total_tickets = $result->total_tickets;
		}
		
		$query = $db->getQuery(true);
        $query->select('SUM(CASE 
        WHEN number_of_tickets = 1 THEN 5
        WHEN number_of_tickets = 3 THEN 10
        WHEN number_of_tickets = 5 THEN 15
        WHEN number_of_tickets = 10 THEN 25
        WHEN number_of_tickets = 20 THEN 50
        WHEN number_of_tickets = 50 THEN 100
        WHEN number_of_tickets = 100 THEN 200
        WHEN number_of_tickets = 200 THEN 400
        END) AS total_raised');
		$query->from('`#__raffle` AS a');
		
		$db->setQuery($query); 
		
		if($result = $db->loadObject()) {
		    $return->total_raised = $result->total_raised;
		}
		
		return $return;
		
	}
	
}
